### Script written during ChessAgent + PythonScriptingAgent Simple experiment

#!/usr/bin/env python3

import argparse
import chess
import sys
from pathlib import Path
from typing import Dict, Optional, Tuple

def validate_fen(fen: str) -> bool:
    """Validate if the provided FEN string is legal."""
    try:
        chess.Board(fen)
        return True
    except ValueError:
        return False

def parse_descriptive_notation(board: chess.Board, move_text: str) -> Optional[chess.Move]:
    """Convert descriptive notation (P-K4) to a chess.Move object."""
    # Mapping of traditional descriptive notation to algebraic squares
    file_map = {'K': 'e', 'Q': 'd', 'B': {'K': 'f', 'Q': 'c'}, 'N': {'K': 'g', 'Q': 'b'}, 'R': {'K': 'h', 'Q': 'a'}}
    rank_map = {'1': '1', '2': '2', '3': '3', '4': '4', '5': '5', '6': '6', '7': '7', '8': '8'}
    
    try:
        # Remove any check or mate symbols
        move_text = move_text.replace('+', '').replace('#', '')
        
        # Handle castling
        if move_text in ['O-O', '0-0']:
            return chess.Move.from_uci('e1g1' if board.turn == chess.WHITE else 'e8g8')
        if move_text in ['O-O-O', '0-0-0']:
            return chess.Move.from_uci('e1c1' if board.turn == chess.WHITE else 'e8c8')
        
        parts = move_text.split('-')
        if len(parts) != 2:
            return None
            
        piece, square = parts
        
        # Convert descriptive notation to algebraic
        if piece == 'P':
            file = square[0].lower()
            rank = rank_map[square[1]]
        else:
            if 'x' in square:
                # Handle captures
                target_square = square.split('x')[1]
            else:
                target_square = square
                
            file = file_map[piece[0]] if isinstance(file_map[piece[0]], str) else file_map[piece[0]]['K']
            rank = rank_map[target_square[1]]
        
        # Find matching legal move
        for move in board.legal_moves:
            if chess.square_name(move.to_square).startswith(file) and \
               chess.square_name(move.to_square).endswith(rank):
                return move
                
    except (KeyError, IndexError):
        return None
    
    return None

def parse_move(board: chess.Board, move_text: str) -> Optional[chess.Move]:
    """Parse move text in various formats and return a chess.Move object."""
    # Try parsing as UCI
    try:
        move = chess.Move.from_uci(move_text)
        if move in board.legal_moves:
            return move
    except ValueError:
        pass
    
    # Try parsing as SAN
    try:
        move = board.parse_san(move_text)
        return move
    except ValueError:
        pass
    
    # Try parsing as descriptive notation
    move = parse_descriptive_notation(board, move_text)
    if move:
        return move
    
    return None

def get_move_details(board: chess.Board, move: chess.Move) -> Dict:
    """Get detailed information about a chess move."""
    details = {
        'piece': chess.piece_name(board.piece_at(move.from_square).piece_type).capitalize(),
        'from_square': chess.square_name(move.from_square),
        'to_square': chess.square_name(move.to_square),
        'is_capture': board.is_capture(move),
        'captured_piece': None,
        'is_check': False,
        'is_checkmate': False,
        'is_castling': board.is_castling(move),
        'is_en_passant': board.is_en_passant(move),
        'is_promotion': move.promotion is not None,
        'promotion_piece': chess.piece_name(move.promotion) if move.promotion else None,
        'uci': move.uci(),
        'san': board.san(move)
    }
    
    # Get captured piece information
    if details['is_capture']:
        captured = board.piece_at(move.to_square)
        if captured:
            details['captured_piece'] = chess.piece_name(captured.piece_type).capitalize()
        elif details['is_en_passant']:
            details['captured_piece'] = 'Pawn'
    
    # Check for check and checkmate
    board_copy = board.copy()
    board_copy.push(move)
    details['is_check'] = board_copy.is_check()
    details['is_checkmate'] = board_copy.is_checkmate()
    
    return details

def format_move_analysis(details: Dict) -> str:
    """Format move details into a readable analysis."""
    lines = [
        "Move Analysis:",
        "=" * 50,
        f"Piece: {details['piece']}",
        f"From: {details['from_square']} To: {details['to_square']}",
        f"UCI notation: {details['uci']}",
        f"SAN notation: {details['san']}",
        "",
        "Move Properties:",
        "-" * 20
    ]
    
    if details['is_capture']:
        lines.append(f"• Captures {details['captured_piece']}")
    if details['is_castling']:
        lines.append("• Castling move")
    if details['is_en_passant']:
        lines.append("• En passant capture")
    if details['is_promotion']:
        lines.append(f"• Pawn promotion to {details['promotion_piece']}")
    if details['is_check']:
        lines.append("• Gives check")
    if details['is_checkmate']:
        lines.append("• Checkmate!")
    
    if not any([details['is_capture'], details['is_castling'], 
                details['is_en_passant'], details['is_promotion'], 
                details['is_check'], details['is_checkmate']]):
        lines.append("• Normal move")
    
    return "\n".join(lines)

def main():
    parser = argparse.ArgumentParser(description="Chess move notation verifier and converter")
    parser.add_argument("--fen", required=True, help="FEN string of the position")
    parser.add_argument("--move", required=True, help="Move in UCI, SAN, or descriptive notation")
    
    args = parser.parse_args()
    
    # Validate FEN
    if not validate_fen(args.fen):
        print("Error: Invalid FEN string provided")
        sys.exit(1)
    
    board = chess.Board(args.fen)
    
    # Parse and validate move
    move = parse_move(board, args.move)
    
    if not move:
        print(f"Error: '{args.move}' is not a valid move in the current position")
        print("\nLegal moves in this position:")
        for legal_move in board.legal_moves:
            print(f"  • {board.san(legal_move)} ({legal_move.uci()})")
        sys.exit(1)
    
    # Get and display move details
    details = get_move_details(board, move)
    print(format_move_analysis(details))

if __name__ == "__main__":
    main()